@@ -17,16 +17,17 @@ package cmd
 
 import (
 	"fmt"
+	"net"
+	"strconv"
+
 	"github.com/osrg/gobgp/config"
 	"github.com/osrg/gobgp/packet/bmp"
 	"github.com/spf13/cobra"
-	"net"
-	"strconv"
 )
 
 func modBmpServer(cmdType string, args []string) error {
 	if len(args) < 1 {
-		return fmt.Errorf("usage: gobgp bmp %s <addr>[:<port>] [{pre|post|both}]", cmdType)
+		return fmt.Errorf("usage: gobgp bmp %s <addr>[:<port>] [{pre|post|both|local-rib|all}]", cmdType)
 	}
 
 	var address string
@@ -49,12 +50,17 @@ func modBmpServer(cmdType string, args []string) error {
 		policyType := config.BMP_ROUTE_MONITORING_POLICY_TYPE_PRE_POLICY
 		if len(args) > 1 {
 			switch args[1] {
+			case "pre":
 			case "post":
 				policyType = config.BMP_ROUTE_MONITORING_POLICY_TYPE_POST_POLICY
 			case "both":
 				policyType = config.BMP_ROUTE_MONITORING_POLICY_TYPE_BOTH
+			case "local-rib":
+				policyType = config.BMP_ROUTE_MONITORING_POLICY_TYPE_LOCAL_RIB
+			case "all":
+				policyType = config.BMP_ROUTE_MONITORING_POLICY_TYPE_ALL
 			default:
-				return fmt.Errorf("invalid bmp policy type. valid type is {pre|post|both}")
+				return fmt.Errorf("invalid bmp policy type. valid type is {pre|post|both|local-rib|all}")
 			}
 		}
 		err = client.AddBMP(&config.BmpServerConfig{
