@@ -45,41 +45,44 @@ const (
 	ENCAP
 	ESI_LABEL
 	ROUTER_MAC
+	DEFAULT_GATEWAY
 	VALID
 	NOT_FOUND
 	INVALID
 )
 
 var ExtCommNameMap = map[ExtCommType]string{
-	ACCEPT:     "accept",
-	DISCARD:    "discard",
-	RATE:       "rate-limit",
-	REDIRECT:   "redirect",
-	MARK:       "mark",
-	ACTION:     "action",
-	RT:         "rt",
-	ENCAP:      "encap",
-	ESI_LABEL:  "esi-label",
-	ROUTER_MAC: "router-mac",
-	VALID:      "valid",
-	NOT_FOUND:  "not-found",
-	INVALID:    "invalid",
+	ACCEPT:          "accept",
+	DISCARD:         "discard",
+	RATE:            "rate-limit",
+	REDIRECT:        "redirect",
+	MARK:            "mark",
+	ACTION:          "action",
+	RT:              "rt",
+	ENCAP:           "encap",
+	ESI_LABEL:       "esi-label",
+	ROUTER_MAC:      "router-mac",
+	DEFAULT_GATEWAY: "default-gateway",
+	VALID:           "valid",
+	NOT_FOUND:       "not-found",
+	INVALID:         "invalid",
 }
 
 var ExtCommValueMap = map[string]ExtCommType{
-	ExtCommNameMap[ACCEPT]:     ACCEPT,
-	ExtCommNameMap[DISCARD]:    DISCARD,
-	ExtCommNameMap[RATE]:       RATE,
-	ExtCommNameMap[REDIRECT]:   REDIRECT,
-	ExtCommNameMap[MARK]:       MARK,
-	ExtCommNameMap[ACTION]:     ACTION,
-	ExtCommNameMap[RT]:         RT,
-	ExtCommNameMap[ENCAP]:      ENCAP,
-	ExtCommNameMap[ESI_LABEL]:  ESI_LABEL,
-	ExtCommNameMap[ROUTER_MAC]: ROUTER_MAC,
-	ExtCommNameMap[VALID]:      VALID,
-	ExtCommNameMap[NOT_FOUND]:  NOT_FOUND,
-	ExtCommNameMap[INVALID]:    INVALID,
+	ExtCommNameMap[ACCEPT]:          ACCEPT,
+	ExtCommNameMap[DISCARD]:         DISCARD,
+	ExtCommNameMap[RATE]:            RATE,
+	ExtCommNameMap[REDIRECT]:        REDIRECT,
+	ExtCommNameMap[MARK]:            MARK,
+	ExtCommNameMap[ACTION]:          ACTION,
+	ExtCommNameMap[RT]:              RT,
+	ExtCommNameMap[ENCAP]:           ENCAP,
+	ExtCommNameMap[ESI_LABEL]:       ESI_LABEL,
+	ExtCommNameMap[ROUTER_MAC]:      ROUTER_MAC,
+	ExtCommNameMap[DEFAULT_GATEWAY]: DEFAULT_GATEWAY,
+	ExtCommNameMap[VALID]:           VALID,
+	ExtCommNameMap[NOT_FOUND]:       NOT_FOUND,
+	ExtCommNameMap[INVALID]:         INVALID,
 }
 
 func rateLimitParser(args []string) ([]bgp.ExtendedCommunityInterface, error) {
@@ -247,6 +250,18 @@ func routerMacParser(args []string) ([]bgp.ExtendedCommunityInterface, error) {
 	return []bgp.ExtendedCommunityInterface{o}, nil
 }
 
+func defaultGatewayParser(args []string) ([]bgp.ExtendedCommunityInterface, error) {
+	if len(args) < 1 || args[0] != ExtCommNameMap[DEFAULT_GATEWAY] {
+		return nil, fmt.Errorf("invalid default-gateway")
+	}
+	o := &bgp.OpaqueExtended{
+		IsTransitive: true,
+		Value:        &bgp.DefaultGatewayExtended{},
+		SubType:      bgp.EC_SUBTYPE_DEFAULT_GATEWAY,
+	}
+	return []bgp.ExtendedCommunityInterface{o}, nil
+}
+
 func validationParser(args []string) ([]bgp.ExtendedCommunityInterface, error) {
 	if len(args) < 1 {
 		return nil, fmt.Errorf("invalid validation state")
@@ -269,19 +284,20 @@ func validationParser(args []string) ([]bgp.ExtendedCommunityInterface, error) {
 }
 
 var ExtCommParserMap = map[ExtCommType]func([]string) ([]bgp.ExtendedCommunityInterface, error){
-	ACCEPT:     nil,
-	DISCARD:    rateLimitParser,
-	RATE:       rateLimitParser,
-	REDIRECT:   redirectParser,
-	MARK:       markParser,
-	ACTION:     actionParser,
-	RT:         rtParser,
-	ENCAP:      encapParser,
-	ESI_LABEL:  esiLabelParser,
-	ROUTER_MAC: routerMacParser,
-	VALID:      validationParser,
-	NOT_FOUND:  validationParser,
-	INVALID:    validationParser,
+	ACCEPT:          nil,
+	DISCARD:         rateLimitParser,
+	RATE:            rateLimitParser,
+	REDIRECT:        redirectParser,
+	MARK:            markParser,
+	ACTION:          actionParser,
+	RT:              rtParser,
+	ENCAP:           encapParser,
+	ESI_LABEL:       esiLabelParser,
+	ROUTER_MAC:      routerMacParser,
+	DEFAULT_GATEWAY: defaultGatewayParser,
+	VALID:           validationParser,
+	NOT_FOUND:       validationParser,
+	INVALID:         validationParser,
 }
 
 func ParseExtendedCommunities(args []string) ([]bgp.ExtendedCommunityInterface, error) {
@@ -443,11 +459,11 @@ func ParseEvpnEthernetAutoDiscoveryArgs(args []string) (bgp.AddrPrefixInterface,
 
 func ParseEvpnMacAdvArgs(args []string) (bgp.AddrPrefixInterface, []string, error) {
 	// Format:
-	// <mac address> <ip address> [esi <esi>] etag <etag> label <label> rd <rd> [rt <rt>...] [encap <encap type>]
+	// <mac address> <ip address> [esi <esi>] etag <etag> label <label> rd <rd> [rt <rt>...] [encap <encap type>] [default-gateway]
 	// or
-	// <mac address> <ip address> <etag> [esi <esi>] label <label> rd <rd> [rt <rt>...] [encap <encap type>]
+	// <mac address> <ip address> <etag> [esi <esi>] label <label> rd <rd> [rt <rt>...] [encap <encap type>] [default-gateway]
 	// or
-	// <mac address> <ip address> <etag> <label> [esi <esi>] rd <rd> [rt <rt>...] [encap <encap type>]
+	// <mac address> <ip address> <etag> <label> [esi <esi>] rd <rd> [rt <rt>...] [encap <encap type>] [default-gateway]
 	req := 6
 	if len(args) < req {
 		return nil, nil, fmt.Errorf("%d args required at least, but got %d", req, len(args))
@@ -529,6 +545,12 @@ func ParseEvpnMacAdvArgs(args []string) (bgp.AddrPrefixInterface, []string, erro
 	if len(m["encap"]) > 0 {
 		extcomms = append(extcomms, "encap", m["encap"][0])
 	}
+	for _, a := range args {
+		if a == "default-gateway" {
+			extcomms = append(extcomms, "default-gateway")
+			break
+		}
+	}
 
 	r := &bgp.EVPNMacIPAdvertisementRoute{
 		RD:               rd,
@@ -1196,41 +1218,74 @@ func modPath(resource string, name, modtype string, args []string) error {
 		if resource == CMD_VRF {
 			cmdstr = fmt.Sprintf("vrf %s", name)
 		}
-
+		rdHelpMsgFmt := `
+    <RD> : xxx:yyy, xxx.xxx.xxx.xxx:yyy, xxx.xxx:yyy`
 		ss := make([]string, 0, len(bgp.ProtocolNameMap))
 		for _, v := range bgp.ProtocolNameMap {
 			ss = append(ss, v)
 		}
-		ss = append(ss, "<VALUE>")
-		protos := strings.Join(ss, ", ")
+		sort.SliceStable(ss, func(i, j int) bool { return ss[i] < ss[j] })
+		ss = append(ss, "<DEC_NUM>")
+		ipProtocols := strings.Join(ss, ", ")
 		ss = make([]string, 0, len(bgp.TCPFlagNameMap))
-		for _, v := range bgp.TCPFlagNameMap {
-			ss = append(ss, v)
+		for _, v := range bgp.TCPSortedFlags {
+			ss = append(ss, bgp.TCPFlagNameMap[v])
 		}
-		flags := strings.Join(ss, ", ")
+		tcpFlags := strings.Join(ss, ", ")
 		ss = make([]string, 0, len(bgp.EthernetTypeNameMap))
 		for _, v := range bgp.EthernetTypeNameMap {
 			ss = append(ss, v)
 		}
+		sort.SliceStable(ss, func(i, j int) bool { return ss[i] < ss[j] })
+		ss = append(ss, "<DEC_NUM>")
 		etherTypes := strings.Join(ss, ", ")
 		helpErrMap := map[bgp.RouteFamily]error{}
 		helpErrMap[bgp.RF_IPv4_UC] = fmt.Errorf("usage: %s rib %s <PREFIX> [identifier <VALUE>] [origin { igp | egp | incomplete }] [aspath <VALUE>] [nexthop <ADDRESS>] [med <VALUE>] [local-pref <VALUE>] [community <VALUE>] [aigp metric <METRIC>] [large-community <VALUE> ] -a ipv4", cmdstr, modtype)
 		helpErrMap[bgp.RF_IPv6_UC] = fmt.Errorf("usage: %s rib %s <PREFIX> [identifier <VALUE>] [origin { igp | egp | incomplete }] [aspath <VALUE>] [nexthop <ADDRESS>] [med <VALUE>] [local-pref <VALUE>] [community <VALUE>] [aigp metric <METRIC>] [large-community <VALUE> ] -a ipv6", cmdstr, modtype)
 		fsHelpMsgFmt := fmt.Sprintf(`err: %s
-usage: %s rib %s%%smatch <MATCH_EXPR> then <THEN_EXPR> -a %%s
-%%s
-    <THEN_EXPR> : { %s | %s | %s <value> | %s <RT> | %s <value> | %s { sample | terminal | sample-terminal } | %s <RT>... }...
-    <RT> : xxx:yyy, xx.xx.xx.xx:yyy, xxx.xxx:yyy`, err, cmdstr, modtype,
-			ExtCommNameMap[ACCEPT], ExtCommNameMap[DISCARD],
-			ExtCommNameMap[RATE], ExtCommNameMap[REDIRECT],
-			ExtCommNameMap[MARK], ExtCommNameMap[ACTION], ExtCommNameMap[RT])
-		ipFsMatchExpr := fmt.Sprintf(`   <MATCH_EXPR> : { %s <PREFIX> [<OFFSET>] | %s <PREFIX> [<OFFSET>] |
-                    %s <PROTO>... | %s [!] [=] <FRAGMENT_TYPE> | %s [!] [=] <TCPFLAG>... |
-                    { %s | %s | %s | %s | %s | %s | %s | %s } <ITEM>... }...
-    <PROTO> : %s
-    <FRAGMENT_TYPE> : dont-fragment, is-fragment, first-fragment, last-fragment, not-a-fragment
-    <TCPFLAG> : %s
-    <ITEM> : & {<|<=|>|>=|==|!=}<value>`,
+usage: %s rib -a %%s %s%%s match <MATCH> then <THEN>%%s%%s%%s
+    <THEN> : { %s |
+               %s |
+               %s <RATE> [as <AS>] |
+               %s <RT> |
+               %s <DEC_NUM> |
+               %s { sample | terminal | sample-terminal } }...
+    <RT> : xxx:yyy, xxx.xxx.xxx.xxx:yyy, xxxx::xxxx:yyy, xxx.xxx:yyy`,
+			err,
+			cmdstr,
+			// <address family>
+			modtype,
+			// "" or " rd <RD>"
+			// "" or " [rt <RT>]"
+			// <help message for RD>
+			// <MATCH>
+			ExtCommNameMap[ACCEPT],
+			ExtCommNameMap[DISCARD],
+			ExtCommNameMap[RATE],
+			ExtCommNameMap[REDIRECT],
+			ExtCommNameMap[MARK],
+			ExtCommNameMap[ACTION],
+		)
+		baseFsMatchExpr := fmt.Sprintf(`
+    <MATCH> : { %s <PREFIX> [<OFFSET>] |
+                %s <PREFIX> [<OFFSET>] |
+                %s <PROTOCOLS>... |
+                %s <FRAGMENTS>... |
+                %s <TCP_FLAGS>... |
+                %s <ITEM>... |
+                %s <ITEM>... |
+                %s <ITEM>... |
+                %s <ITEM>... |
+                %s <ITEM>... |
+                %s <ITEM>... |
+                %s <ITEM>... %%s}...
+    <PROTOCOLS> : [&] [<|<=|>|>=|==|!=] <PROTOCOL>
+    <PROTOCOL> : %s
+    <FRAGMENTS> : [&] [=|!|!=] <FRAGMENT>
+    <FRAGMENT> : dont-fragment, is-fragment, first-fragment, last-fragment, not-a-fragment
+    <TCP_FLAGS> : [&] [=|!|!=] <TCP_FLAG>
+    <TCP_FLAG> : %s%%s
+    <ITEM> : [&] [<|<=|>|>=|==|!=] <DEC_NUM>`,
 			bgp.FlowSpecNameMap[bgp.FLOW_SPEC_TYPE_DST_PREFIX],
 			bgp.FlowSpecNameMap[bgp.FLOW_SPEC_TYPE_SRC_PREFIX],
 			bgp.FlowSpecNameMap[bgp.FLOW_SPEC_TYPE_IP_PROTO],
@@ -1243,17 +1298,29 @@ usage: %s rib %s%%smatch <MATCH_EXPR> then <THEN_EXPR> -a %%s
 			bgp.FlowSpecNameMap[bgp.FLOW_SPEC_TYPE_ICMP_CODE],
 			bgp.FlowSpecNameMap[bgp.FLOW_SPEC_TYPE_PKT_LEN],
 			bgp.FlowSpecNameMap[bgp.FLOW_SPEC_TYPE_DSCP],
-			bgp.FlowSpecNameMap[bgp.FLOW_SPEC_TYPE_LABEL],
-			protos,
-			flags,
+			// <additional help messages if exists>
+			ipProtocols,
+			tcpFlags,
+			// <additional help messages if exists>
 		)
-		helpErrMap[bgp.RF_FS_IPv4_UC] = fmt.Errorf(fsHelpMsgFmt, " ", "ipv4-flowspec", ipFsMatchExpr)
-		helpErrMap[bgp.RF_FS_IPv6_UC] = fmt.Errorf(fsHelpMsgFmt, " ", "ipv6-flowspec", ipFsMatchExpr)
-		helpErrMap[bgp.RF_FS_IPv4_VPN] = fmt.Errorf(fsHelpMsgFmt, " rd <RD> ", "ipv4-l3vpn-flowspec", ipFsMatchExpr)
-		helpErrMap[bgp.RF_FS_IPv6_VPN] = fmt.Errorf(fsHelpMsgFmt, " rd <RD> ", "ipv6-l3vpn-flowspec", ipFsMatchExpr)
-		macFsMatchExpr := fmt.Sprintf(`   <MATCH_EXPR> : { { %s | %s } <MAC> | %s <ETHER_TYPE> | { %s | %s | %s | %s | %s | %s | %s | %s } <ITEM>... }...
-    <ETHER_TYPE> : %s
-    <ITEM> : &?{<|>|=}<value>`,
+		ipv4FsMatchExpr := fmt.Sprintf(baseFsMatchExpr, "", "")
+		ipv6FsMatchExpr := fmt.Sprintf(baseFsMatchExpr, fmt.Sprintf(`|
+                %s <ITEM>... `,
+			bgp.FlowSpecNameMap[bgp.FLOW_SPEC_TYPE_LABEL]), "")
+		l2vpnFsMatchExpr := fmt.Sprintf(baseFsMatchExpr, fmt.Sprintf(`|
+                %s <ITEM>... |
+                %s <MAC_ADDRESS> |
+                %s <MAC_ADDRESS> |
+                %s <ETHER_TYPES>... |
+                %s <ITEM>... |
+                %s <ITEM>... |
+                %s <ITEM>... |
+                %s <ITEM>... |
+                %s <ITEM>... |
+                %s <ITEM>... |
+                %s <ITEM>... |
+                %s <ITEM>... `,
+			bgp.FlowSpecNameMap[bgp.FLOW_SPEC_TYPE_LABEL],
 			bgp.FlowSpecNameMap[bgp.FLOW_SPEC_TYPE_DST_MAC],
 			bgp.FlowSpecNameMap[bgp.FLOW_SPEC_TYPE_SRC_MAC],
 			bgp.FlowSpecNameMap[bgp.FLOW_SPEC_TYPE_ETHERNET_TYPE],
@@ -1264,13 +1331,18 @@ usage: %s rib %s%%smatch <MATCH_EXPR> then <THEN_EXPR> -a %%s
 			bgp.FlowSpecNameMap[bgp.FLOW_SPEC_TYPE_VID],
 			bgp.FlowSpecNameMap[bgp.FLOW_SPEC_TYPE_COS],
 			bgp.FlowSpecNameMap[bgp.FLOW_SPEC_TYPE_INNER_VID],
-			bgp.FlowSpecNameMap[bgp.FLOW_SPEC_TYPE_INNER_COS],
-			etherTypes,
-		)
-		helpErrMap[bgp.RF_FS_L2_VPN] = fmt.Errorf(fsHelpMsgFmt, "l2vpn-flowspec", macFsMatchExpr)
+			bgp.FlowSpecNameMap[bgp.FLOW_SPEC_TYPE_INNER_COS]), fmt.Sprintf(`
+    <ETHER_TYPES> : [&] [<|<=|>|>=|==|!=] <ETHER_TYPE>
+    <ETHER_TYPE> : %s`,
+			etherTypes))
+		helpErrMap[bgp.RF_FS_IPv4_UC] = fmt.Errorf(fsHelpMsgFmt, "ipv4-flowspec", "", "", "", ipv4FsMatchExpr)
+		helpErrMap[bgp.RF_FS_IPv6_UC] = fmt.Errorf(fsHelpMsgFmt, "ipv6-flowspec", "", "", "", ipv6FsMatchExpr)
+		helpErrMap[bgp.RF_FS_IPv4_VPN] = fmt.Errorf(fsHelpMsgFmt, "ipv4-l3vpn-flowspec", " rd <RD>", " [rt <RT>]", rdHelpMsgFmt, ipv4FsMatchExpr)
+		helpErrMap[bgp.RF_FS_IPv6_VPN] = fmt.Errorf(fsHelpMsgFmt, "ipv6-l3vpn-flowspec", " rd <RD>", " [rt <RT>]", rdHelpMsgFmt, ipv6FsMatchExpr)
+		helpErrMap[bgp.RF_FS_L2_VPN] = fmt.Errorf(fsHelpMsgFmt, "l2vpn-flowspec", " rd <RD>", " [rt <RT>]", rdHelpMsgFmt, l2vpnFsMatchExpr)
 		helpErrMap[bgp.RF_EVPN] = fmt.Errorf(`usage: %s rib %s { a-d <A-D> | macadv <MACADV> | multicast <MULTICAST> | esi <ESI> | prefix <PREFIX> } -a evpn
     <A-D>       : esi <esi> etag <etag> label <label> rd <rd> [rt <rt>...] [encap <encap type>] [esi-label <esi-label> [single-active | all-active]]
-    <MACADV>    : <mac address> <ip address> [esi <esi>] etag <etag> label <label> rd <rd> [rt <rt>...] [encap <encap type>]
+    <MACADV>    : <mac address> <ip address> [esi <esi>] etag <etag> label <label> rd <rd> [rt <rt>...] [encap <encap type>] [default-gateway]
     <MULTICAST> : <ip address> etag <etag> rd <rd> [rt <rt>...] [encap <encap type>]
     <ESI>       : <ip address> esi <esi> rd <rd> [rt <rt>...] [encap <encap type>]
     <PREFIX>    : <ip prefix> [gw <gateway>] [esi <esi>] etag <etag> [label <label>] rd <rd> [rt <rt>...] [encap <encap type>] [router-mac <mac address>]`, cmdstr, modtype)
