@@ -27,21 +27,26 @@ import (
 	"github.com/spf13/cobra"
 )
 
-func injectMrt(filename string, count int, skip int, onlyBest bool) error {
+func injectMrt() error {
 
-	file, err := os.Open(filename)
+	file, err := os.Open(mrtOpts.Filename)
 	if err != nil {
 		return fmt.Errorf("failed to open file: %s", err)
 	}
 
+	if mrtOpts.NextHop != nil && !mrtOpts.SkipV4 && !mrtOpts.SkipV6 {
+		fmt.Println("You should probably specify either --no-ipv4 or --no-ipv6 when overwriting nexthop, unless your dump contains only one type of routes")
+	}
+
 	idx := 0
+	if mrtOpts.QueueSize < 1 {
+		return fmt.Errorf("Specified queue size is smaller than 1, refusing to run with unbounded memory usage")
+	}
 
-	ch := make(chan []*table.Path, 1<<20)
-
+	ch := make(chan []*table.Path, mrtOpts.QueueSize)
 	go func() {
 
 		var peers []*mrt.Peer
-
 		for {
 			buf := make([]byte, mrt.MRT_COMMON_HEADER_LEN)
 			_, err := file.Read(buf)
@@ -79,7 +84,14 @@ func injectMrt(filename string, count int, skip int, onlyBest bool) error {
 				case mrt.PEER_INDEX_TABLE:
 					peers = msg.Body.(*mrt.PeerIndexTable).Peers
 					continue
-				case mrt.RIB_IPV4_UNICAST, mrt.RIB_IPV6_UNICAST:
+				case mrt.RIB_IPV4_UNICAST, mrt.RIB_IPV4_UNICAST_ADDPATH:
+					if mrtOpts.SkipV4 {
+						continue
+					}
+				case mrt.RIB_IPV6_UNICAST, mrt.RIB_IPV6_UNICAST_ADDPATH:
+					if mrtOpts.SkipV6 {
+						continue
+					}
 				case mrt.GEO_PEER_TABLE:
 					fmt.Printf("WARNING: Skipping GEO_PEER_TABLE: %s", msg.Body.(*mrt.GeoPeerTable))
 				default:
@@ -106,8 +118,13 @@ func injectMrt(filename string, count int, skip int, onlyBest bool) error {
 					t := time.Unix(int64(e.OriginatedTime), 0)
 					paths = append(paths, table.NewPath(source, nlri, false, e.PathAttributes, t, false))
 				}
+				if mrtOpts.NextHop != nil {
+					for _, p := range paths {
+						p.SetNexthop(mrtOpts.NextHop)
+					}
+				}
 
-				if onlyBest {
+				if mrtOpts.Best {
 					dst := table.NewDestination(nlri, 0)
 					for _, p := range paths {
 						dst.AddNewPath(p)
@@ -119,12 +136,12 @@ func injectMrt(filename string, count int, skip int, onlyBest bool) error {
 					paths = []*table.Path{best}
 				}
 
-				if idx >= skip {
+				if idx >= mrtOpts.RecordSkip {
 					ch <- paths
 				}
 
 				idx += 1
-				if idx == count+skip {
+				if idx == mrtOpts.RecordCount+mrtOpts.RecordSkip {
 					break
 				}
 			}
@@ -135,7 +152,7 @@ func injectMrt(filename string, count int, skip int, onlyBest bool) error {
 
 	stream, err := client.AddPathByStream()
 	if err != nil {
-		return fmt.Errorf("failed to modpath: %s", err)
+		return fmt.Errorf("failed to add path: %s", err)
 	}
 
 	for paths := range ch {
@@ -158,23 +175,24 @@ func NewMrtCmd() *cobra.Command {
 			if len(args) < 1 {
 				exitWithError(fmt.Errorf("usage: gobgp mrt inject global <filename> [<count> [<skip>]]"))
 			}
-			filename := args[0]
-			count := -1
-			skip := 0
+			mrtOpts.Filename = args[0]
 			if len(args) > 1 {
 				var err error
-				count, err = strconv.Atoi(args[1])
+				mrtOpts.RecordCount, err = strconv.Atoi(args[1])
 				if err != nil {
 					exitWithError(fmt.Errorf("invalid count value: %s", args[1]))
 				}
 				if len(args) > 2 {
-					skip, err = strconv.Atoi(args[2])
+					mrtOpts.RecordSkip, err = strconv.Atoi(args[2])
 					if err != nil {
 						exitWithError(fmt.Errorf("invalid skip value: %s", args[2]))
 					}
 				}
+			} else {
+				mrtOpts.RecordCount = -1
+				mrtOpts.RecordSkip = 0
 			}
-			err := injectMrt(filename, count, skip, mrtOpts.Best)
+			err := injectMrt()
 			if err != nil {
 				exitWithError(err)
 			}
@@ -192,5 +210,9 @@ func NewMrtCmd() *cobra.Command {
 	mrtCmd.AddCommand(injectCmd)
 
 	mrtCmd.PersistentFlags().BoolVarP(&mrtOpts.Best, "only-best", "", false, "inject only best paths")
+	mrtCmd.PersistentFlags().BoolVarP(&mrtOpts.SkipV4, "no-ipv4", "", false, "Do not import IPv4 routes")
+	mrtCmd.PersistentFlags().BoolVarP(&mrtOpts.SkipV6, "no-ipv6", "", false, "Do not import IPv6 routes")
+	mrtCmd.PersistentFlags().IntVarP(&mrtOpts.QueueSize, "queue-size", "", 1<<10, "Maximum number of updates to keep queued")
+	mrtCmd.PersistentFlags().IPVarP(&mrtOpts.NextHop, "nexthop", "", nil, "Overwrite nexthop")
 	return mrtCmd
 }
