@@ -714,8 +714,13 @@ func showNeighborRib(r string, name string, args []string) error {
 	var filter []*table.LookupPrefix
 	if len(args) > 0 {
 		target := args[0]
-		if _, _, err = parseCIDRorIP(args[0]); err != nil {
-			return err
+		switch family {
+		case bgp.RF_EVPN:
+			// Uses target as EVPN Route Type string
+		default:
+			if _, _, err = parseCIDRorIP(target); err != nil {
+				return err
+			}
 		}
 		var option table.LookupOption
 		args = args[1:]
@@ -992,6 +997,11 @@ func modNeighbor(cmdType string, args []string) error {
 		}
 		if unnumbered {
 			peer.Config.NeighborInterface = m["interface"][0]
+			addr, err := config.GetIPv6LinkLocalNeighborAddress(peer.Config.NeighborInterface)
+			if err != nil {
+				return nil, err
+			}
+			peer.State.NeighborAddress = addr
 		} else {
 			peer.Config.NeighborAddress = m[""][0]
 			peer.State.NeighborAddress = m[""][0]
