@@ -32,7 +32,7 @@ type pathList []*table.Path
 
 type nexthopTrackingManager struct {
 	dead              chan struct{}
-	nexthopCache      []*net.IP
+	nexthopCache      map[string]struct{}
 	server            *BgpServer
 	delay             int
 	isScheduled       bool
@@ -44,7 +44,7 @@ type nexthopTrackingManager struct {
 func newNexthopTrackingManager(server *BgpServer, delay int) *nexthopTrackingManager {
 	return &nexthopTrackingManager{
 		dead:              make(chan struct{}),
-		nexthopCache:      make([]*net.IP, 0),
+		nexthopCache:      make(map[string]struct{}),
 		server:            server,
 		delay:             delay,
 		scheduledPathList: make(map[string]pathList, 0),
@@ -60,22 +60,25 @@ func (m *nexthopTrackingManager) stop() {
 }
 
 func (m *nexthopTrackingManager) isRegisteredNexthop(nexthop net.IP) bool {
-	for _, cached := range m.nexthopCache {
-		if cached.Equal(nexthop) {
-			return true
-		}
-	}
-	return false
+	key := nexthop.String()
+	_, ok := m.nexthopCache[key]
+	return ok
 }
 
 func (m *nexthopTrackingManager) registerNexthop(nexthop net.IP) bool {
-	if m.isRegisteredNexthop(nexthop) {
+	key := nexthop.String()
+	if _, ok := m.nexthopCache[key]; ok {
 		return false
 	}
-	m.nexthopCache = append(m.nexthopCache, &nexthop)
+	m.nexthopCache[key] = struct{}{}
 	return true
 }
 
+func (m *nexthopTrackingManager) unregisterNexthop(nexthop net.IP) {
+	key := nexthop.String()
+	delete(m.nexthopCache, key)
+}
+
 func (m *nexthopTrackingManager) appendPathList(paths pathList) {
 	if len(paths) == 0 {
 		return
@@ -123,7 +126,7 @@ func (m *nexthopTrackingManager) loop() {
 			log.WithFields(log.Fields{
 				"Topic": "Zebra",
 				"Event": "Nexthop Tracking",
-			}).Debugf("penalty 500 chrged: penalty: %d", penalty)
+			}).Debugf("penalty 500 charged: penalty: %d", penalty)
 
 			m.appendPathList(paths)
 
@@ -177,10 +180,24 @@ func (m *nexthopTrackingManager) scheduleUpdate(paths pathList) {
 	m.pathListCh <- paths
 }
 
-func filterOutNilPath(paths pathList) pathList {
+func (m *nexthopTrackingManager) filterPathToRegister(paths pathList) pathList {
 	filteredPaths := make(pathList, 0, len(paths))
 	for _, path := range paths {
-		if path == nil {
+		if path == nil || path.IsFromExternal() {
+			continue
+		}
+		// NEXTHOP_UNREGISTER message will be sent when GoBGP received
+		// NEXTHOP_UPDATE message and there is no path bound for the updated
+		// nexthop.
+		// Here filters out withdraw paths and paths whose nexthop is:
+		// - already invalidated
+		// - already registered
+		// - unspecified address
+		if path.IsWithdraw || path.IsNexthopInvalid {
+			continue
+		}
+		nexthop := path.GetNexthop()
+		if m.isRegisteredNexthop(nexthop) || nexthop.IsUnspecified() {
 			continue
 		}
 		filteredPaths = append(filteredPaths, path)
@@ -199,24 +216,18 @@ func filterOutExternalPath(paths pathList) pathList {
 	return filteredPaths
 }
 
-func newIPRouteMessage(dst pathList, version uint8, vrfId uint16) *zebra.Message {
+func newIPRouteBody(dst pathList) (body *zebra.IPRouteBody, isWithdraw bool) {
 	paths := filterOutExternalPath(dst)
 	if len(paths) == 0 {
-		return nil
+		return nil, false
 	}
 	path := paths[0]
 
 	l := strings.SplitN(path.GetNlri().String(), "/", 2)
-	var command zebra.API_TYPE
 	var prefix net.IP
 	nexthops := make([]net.IP, 0, len(paths))
 	switch path.GetRouteFamily() {
 	case bgp.RF_IPv4_UC, bgp.RF_IPv4_VPN:
-		if path.IsWithdraw == true {
-			command = zebra.IPV4_ROUTE_DELETE
-		} else {
-			command = zebra.IPV4_ROUTE_ADD
-		}
 		if path.GetRouteFamily() == bgp.RF_IPv4_UC {
 			prefix = path.GetNlri().(*bgp.IPAddrPrefix).IPAddrPrefixDefault.Prefix.To4()
 		} else {
@@ -226,11 +237,6 @@ func newIPRouteMessage(dst pathList, version uint8, vrfId uint16) *zebra.Message
 			nexthops = append(nexthops, p.GetNexthop().To4())
 		}
 	case bgp.RF_IPv6_UC, bgp.RF_IPv6_VPN:
-		if path.IsWithdraw == true {
-			command = zebra.IPV6_ROUTE_DELETE
-		} else {
-			command = zebra.IPV6_ROUTE_ADD
-		}
 		if path.GetRouteFamily() == bgp.RF_IPv6_UC {
 			prefix = path.GetNlri().(*bgp.IPv6AddrPrefix).IPAddrPrefixDefault.Prefix.To16()
 		} else {
@@ -240,7 +246,7 @@ func newIPRouteMessage(dst pathList, version uint8, vrfId uint16) *zebra.Message
 			nexthops = append(nexthops, p.GetNexthop().To16())
 		}
 	default:
-		return nil
+		return nil, false
 	}
 	msgFlags := zebra.MESSAGE_NEXTHOP
 	plen, _ := strconv.Atoi(l[1])
@@ -255,64 +261,42 @@ func newIPRouteMessage(dst pathList, version uint8, vrfId uint16) *zebra.Message
 	} else if info.MultihopTtl > 0 {
 		flags = zebra.FLAG_INTERNAL
 	}
-	return &zebra.Message{
-		Header: zebra.Header{
-			Len:     zebra.HeaderSize(version),
-			Marker:  zebra.HEADER_MARKER,
-			Version: version,
-			Command: command,
-			VrfId:   vrfId,
-		},
-		Body: &zebra.IPRouteBody{
-			Type:         zebra.ROUTE_BGP,
-			Flags:        flags,
-			SAFI:         zebra.SAFI_UNICAST,
-			Message:      msgFlags,
-			Prefix:       prefix,
-			PrefixLength: uint8(plen),
-			Nexthops:     nexthops,
-			Metric:       med,
-		},
-	}
+	return &zebra.IPRouteBody{
+		Type:         zebra.ROUTE_BGP,
+		Flags:        flags,
+		SAFI:         zebra.SAFI_UNICAST,
+		Message:      msgFlags,
+		Prefix:       prefix,
+		PrefixLength: uint8(plen),
+		Nexthops:     nexthops,
+		Metric:       med,
+	}, path.IsWithdraw
 }
 
-func newNexthopRegisterMessage(dst pathList, version uint8, vrfId uint16, nhtManager *nexthopTrackingManager) *zebra.Message {
-	// Note: NEXTHOP_REGISTER and NEXTHOP_UNREGISTER messages are not
-	// supported in Zebra protocol version<3.
-	if version < 3 || nhtManager == nil {
-		return nil
+func newNexthopRegisterBody(dst pathList, nhtManager *nexthopTrackingManager) (body *zebra.NexthopRegisterBody, isWithdraw bool) {
+	if nhtManager == nil {
+		return nil, false
 	}
 
-	paths := filterOutNilPath(dst)
+	paths := nhtManager.filterPathToRegister(dst)
 	if len(paths) == 0 {
-		return nil
+		return nil, false
 	}
+	path := paths[0]
 
-	route_family := paths[0].GetRouteFamily()
-	command := zebra.NEXTHOP_REGISTER
-	if paths[0].IsWithdraw == true {
-		// TODO:
-		// Send NEXTHOP_UNREGISTER message if the given nexthop is no longer
-		// referred by any path. Currently, do not send NEXTHOP_UNREGISTER
-		// message to simplify the implementation.
-		//command = zebra.NEXTHOP_UNREGISTER
-		return nil
+	if path.IsWithdraw == true {
+		// NEXTHOP_UNREGISTER message will be sent when GoBGP received
+		// NEXTHOP_UPDATE message and there is no path bound for the updated
+		// nexthop. So there is nothing to do here.
+		return nil, true
 	}
 
+	family := path.GetRouteFamily()
 	nexthops := make([]*zebra.RegisteredNexthop, 0, len(paths))
 	for _, p := range paths {
 		nexthop := p.GetNexthop()
-		// Skips to register or unregister the given nexthop
-		// when the nexthop is:
-		// - already registered
-		// - already invalidated
-		// - an unspecified address
-		if nhtManager.isRegisteredNexthop(nexthop) || p.IsNexthopInvalid || nexthop.IsUnspecified() {
-			continue
-		}
-
 		var nh *zebra.RegisteredNexthop
-		switch route_family {
+		switch family {
 		case bgp.RF_IPv4_UC, bgp.RF_IPv4_VPN:
 			nh = &zebra.RegisteredNexthop{
 				Family: syscall.AF_INET,
@@ -324,7 +308,7 @@ func newNexthopRegisterMessage(dst pathList, version uint8, vrfId uint16, nhtMan
 				Prefix: nexthop.To16(),
 			}
 		default:
-			return nil
+			continue
 		}
 		nexthops = append(nexthops, nh)
 		nhtManager.registerNexthop(nexthop)
@@ -333,36 +317,22 @@ func newNexthopRegisterMessage(dst pathList, version uint8, vrfId uint16, nhtMan
 	// If no nexthop needs to be registered or unregistered,
 	// skips to send message.
 	if len(nexthops) == 0 {
-		return nil
+		return nil, path.IsWithdraw
 	}
 
-	return &zebra.Message{
-		Header: zebra.Header{
-			Len:     zebra.HeaderSize(version),
-			Marker:  zebra.HEADER_MARKER,
-			Version: version,
-			Command: command,
-			VrfId:   vrfId,
-		},
-		Body: &zebra.NexthopRegisterBody{
-			Nexthops: nexthops,
-		},
-	}
+	return &zebra.NexthopRegisterBody{
+		Nexthops: nexthops,
+	}, path.IsWithdraw
 }
 
 func createPathFromIPRouteMessage(m *zebra.Message) *table.Path {
-
 	header := m.Header
 	body := m.Body.(*zebra.IPRouteBody)
-	family := bgp.RF_IPv6_UC
-	if header.Command == zebra.IPV4_ROUTE_ADD || header.Command == zebra.IPV4_ROUTE_DELETE {
-		family = bgp.RF_IPv4_UC
-	}
+	family := body.RouteFamily()
+	isWithdraw := body.IsWithdraw()
 
 	var nlri bgp.AddrPrefixInterface
 	pattr := make([]bgp.PathAttributeInterface, 0)
-	var isWithdraw bool = header.Command == zebra.IPV4_ROUTE_DELETE || header.Command == zebra.IPV6_ROUTE_DELETE
-
 	origin := bgp.NewPathAttributeOrigin(bgp.BGP_ORIGIN_ATTR_TYPE_IGP)
 	pattr = append(pattr, origin)
 
@@ -409,7 +379,7 @@ func createPathFromIPRouteMessage(m *zebra.Message) *table.Path {
 	return path
 }
 
-func createPathListFromNexthopUpdateMessage(m *zebra.Message, manager *table.TableManager) (pathList, error) {
+func createPathListFromNexthopUpdateMessage(m *zebra.Message, manager *table.TableManager, nhtManager *nexthopTrackingManager) (pathList, *zebra.NexthopRegisterBody, error) {
 	body := m.Body.(*zebra.NexthopUpdateBody)
 	isNexthopInvalid := len(body.Nexthops) == 0
 
@@ -420,11 +390,26 @@ func createPathListFromNexthopUpdateMessage(m *zebra.Message, manager *table.Tab
 	case uint16(syscall.AF_INET6):
 		rfList = []bgp.RouteFamily{bgp.RF_IPv6_UC, bgp.RF_IPv6_VPN}
 	default:
-		return nil, fmt.Errorf("invalid address family: %d", body.Family)
+		return nil, nil, fmt.Errorf("invalid address family: %d", body.Family)
 	}
 
 	paths := manager.GetPathListWithNexthop(table.GLOBAL_RIB_NAME, rfList, body.Prefix)
-	updatedPathList := make(pathList, 0, len(paths))
+	pathsLen := len(paths)
+
+	// If there is no path bound for the updated nexthop, send
+	// NEXTHOP_UNREGISTER message.
+	var nexthopUnregisterBody *zebra.NexthopRegisterBody
+	if pathsLen == 0 {
+		nexthopUnregisterBody = &zebra.NexthopRegisterBody{
+			Nexthops: []*zebra.RegisteredNexthop{{
+				Family: body.Family,
+				Prefix: body.Prefix,
+			}},
+		}
+		nhtManager.unregisterNexthop(body.Prefix)
+	}
+
+	updatedPathList := make(pathList, 0, pathsLen)
 	for _, path := range paths {
 		newPath := path.Clone(false)
 		if isNexthopInvalid {
@@ -440,7 +425,7 @@ func createPathListFromNexthopUpdateMessage(m *zebra.Message, manager *table.Tab
 		updatedPathList = append(updatedPathList, newPath)
 	}
 
-	return updatedPathList, nil
+	return updatedPathList, nexthopUnregisterBody, nil
 }
 
 type zebraClient struct {
@@ -455,7 +440,10 @@ func (z *zebraClient) stop() {
 }
 
 func (z *zebraClient) loop() {
-	w := z.server.Watch(WatchBestPath(true))
+	w := z.server.Watch([]WatchOption{
+		WatchBestPath(true),
+		WatchPostUpdate(true),
+	}...)
 	defer w.Stop()
 
 	if z.nhtManager != nil {
@@ -468,7 +456,7 @@ func (z *zebraClient) loop() {
 		case <-z.dead:
 			return
 		case msg := <-z.client.Receive():
-			switch msg.Body.(type) {
+			switch body := msg.Body.(type) {
 			case *zebra.IPRouteBody:
 				if p := createPathFromIPRouteMessage(msg); p != nil {
 					if _, err := z.server.AddPath("", pathList{p}); err != nil {
@@ -477,38 +465,46 @@ func (z *zebraClient) loop() {
 				}
 			case *zebra.NexthopUpdateBody:
 				if z.nhtManager != nil {
-					body := msg.Body.(*zebra.NexthopUpdateBody)
-					if paths, err := createPathListFromNexthopUpdateMessage(msg, z.server.globalRib); err != nil {
+					if paths, b, err := createPathListFromNexthopUpdateMessage(msg, z.server.globalRib, z.nhtManager); err != nil {
 						log.Errorf("failed to create updated path list related to nexthop %s", body.Prefix.String())
 					} else {
 						z.nhtManager.scheduleUpdate(paths)
+						if b != nil {
+							z.client.SendNexthopRegister(msg.Header.VrfId, b, true)
+						}
 					}
 				}
 			}
 		case ev := <-w.Event():
-			msg := ev.(*WatchEventBestPath)
-			if table.UseMultiplePaths.Enabled {
-				for _, dst := range msg.MultiPathList {
-					if m := newIPRouteMessage(dst, z.client.Version, 0); m != nil {
-						z.client.Send(m)
+			switch msg := ev.(type) {
+			case *WatchEventBestPath:
+				if table.UseMultiplePaths.Enabled {
+					for _, dst := range msg.MultiPathList {
+						if body, isWithdraw := newIPRouteBody(dst); body != nil {
+							z.client.SendIPRoute(0, body, isWithdraw)
+						}
+						if body, isWithdraw := newNexthopRegisterBody(dst, z.nhtManager); body != nil {
+							z.client.SendNexthopRegister(0, body, isWithdraw)
+						}
 					}
-					if m := newNexthopRegisterMessage(dst, z.client.Version, 0, z.nhtManager); m != nil {
-						z.client.Send(m)
+				} else {
+					for _, path := range msg.PathList {
+						if len(path.VrfIds) == 0 {
+							path.VrfIds = []uint16{0}
+						}
+						for _, i := range path.VrfIds {
+							if body, isWithdraw := newIPRouteBody(pathList{path}); body != nil {
+								z.client.SendIPRoute(i, body, isWithdraw)
+							}
+							if body, isWithdraw := newNexthopRegisterBody(pathList{path}, z.nhtManager); body != nil {
+								z.client.SendNexthopRegister(i, body, isWithdraw)
+							}
+						}
 					}
 				}
-			} else {
-				for _, path := range msg.PathList {
-					if len(path.VrfIds) == 0 {
-						path.VrfIds = []uint16{0}
-					}
-					for _, i := range path.VrfIds {
-						if m := newIPRouteMessage(pathList{path}, z.client.Version, i); m != nil {
-							z.client.Send(m)
-						}
-						if m := newNexthopRegisterMessage(pathList{path}, z.client.Version, i, z.nhtManager); m != nil {
-							z.client.Send(m)
-						}
-					}
+			case *WatchEventUpdate:
+				if body, isWithdraw := newNexthopRegisterBody(msg.PathList, z.nhtManager); body != nil {
+					z.client.SendNexthopRegister(0, body, isWithdraw)
 				}
 			}
 		}
@@ -520,20 +516,20 @@ func newZebraClient(s *BgpServer, url string, protos []string, version uint8, nh
 	if len(l) != 2 {
 		return nil, fmt.Errorf("unsupported url: %s", url)
 	}
-	cli, err := zebra.NewClient(l[0], l[1], zebra.ROUTE_BGP, version)
-	if err != nil {
-		// Retry with another Zebra message version
-		var retry_version uint8 = 2
-		if version == 2 {
-			retry_version = 3
+	var cli *zebra.Client
+	var err error
+	for _, ver := range []uint8{version, 2, 3, 4} {
+		cli, err = zebra.NewClient(l[0], l[1], zebra.ROUTE_BGP, ver)
+		if err == nil {
+			break
 		}
+		// Retry with another Zebra message version
 		log.WithFields(log.Fields{
 			"Topic": "Zebra",
-		}).Warnf("cannot connect to Zebra with message version %d. retry with version %d", version, retry_version)
-		cli, err = zebra.NewClient(l[0], l[1], zebra.ROUTE_BGP, retry_version)
-		if err != nil {
-			return nil, err
-		}
+		}).Warnf("cannot connect to Zebra with message version %d. going to retry another version...", ver)
+	}
+	if cli == nil {
+		return nil, err
 	}
 	// Note: HELLO/ROUTER_ID_ADD messages are automatically sent to negotiate
 	// the Zebra message version in zebra.NewClient().
