@@ -22,6 +22,8 @@ import (
 	"math"
 	"net"
 	"sort"
+	"strconv"
+	"strings"
 	"time"
 
 	log "github.com/sirupsen/logrus"
@@ -136,24 +138,11 @@ type Validation struct {
 	UnmatchedLength []*ROA
 }
 
-type FlowSpecComponents []bgp.FlowSpecComponentInterface
-
-func (c FlowSpecComponents) Len() int {
-	return len(c)
-}
-
-func (c FlowSpecComponents) Swap(i, j int) {
-	c[i], c[j] = c[j], c[i]
-}
-
-func (c FlowSpecComponents) Less(i, j int) bool {
-	return c[i].Type() < c[j].Type()
-}
-
 type Path struct {
 	info       *originInfo
 	IsWithdraw bool
 	pathAttrs  []bgp.PathAttributeInterface
+	attrsHash  uint32
 	reason     BestPathReason
 	parent     *Path
 	dels       []bgp.BGPAttrType
@@ -168,29 +157,10 @@ func NewPath(source *PeerInfo, nlri bgp.AddrPrefixInterface, isWithdraw bool, pa
 		log.WithFields(log.Fields{
 			"Topic": "Table",
 			"Key":   nlri.String(),
-		}).Error("Need to provide patattrs for the path that is not withdraw.")
+		}).Error("Need to provide path attributes for non-withdrawn path.")
 		return nil
 	}
 
-	if nlri != nil && (nlri.SAFI() == bgp.SAFI_FLOW_SPEC_UNICAST || nlri.SAFI() == bgp.SAFI_FLOW_SPEC_VPN) {
-		var coms FlowSpecComponents
-		var f *bgp.FlowSpecNLRI
-		switch nlri.(type) {
-		case *bgp.FlowSpecIPv4Unicast:
-			f = &nlri.(*bgp.FlowSpecIPv4Unicast).FlowSpecNLRI
-		case *bgp.FlowSpecIPv4VPN:
-			f = &nlri.(*bgp.FlowSpecIPv4VPN).FlowSpecNLRI
-		case *bgp.FlowSpecIPv6Unicast:
-			f = &nlri.(*bgp.FlowSpecIPv6Unicast).FlowSpecNLRI
-		case *bgp.FlowSpecIPv6VPN:
-			f = &nlri.(*bgp.FlowSpecIPv6VPN).FlowSpecNLRI
-		}
-		if f != nil {
-			coms = f.Value
-			sort.Sort(coms)
-		}
-	}
-
 	return &Path{
 		info: &originInfo{
 			nlri:               nlri,
@@ -257,13 +227,10 @@ func UpdatePathAttrs(global *config.Global, peer *config.Neighbor, info *PeerInf
 	}
 
 	localAddress := info.LocalAddress
-	isZero := func(ip net.IP) bool {
-		return ip.Equal(net.ParseIP("0.0.0.0")) || ip.Equal(net.ParseIP("::"))
-	}
 	nexthop := path.GetNexthop()
 	if peer.State.PeerType == config.PEER_TYPE_EXTERNAL {
 		// NEXTHOP handling
-		if !path.IsLocal() || isZero(nexthop) {
+		if !path.IsLocal() || nexthop.IsUnspecified() {
 			path.SetNexthop(localAddress)
 		}
 
@@ -271,7 +238,11 @@ func UpdatePathAttrs(global *config.Global, peer *config.Neighbor, info *PeerInf
 		path.RemovePrivateAS(peer.Config.LocalAs, peer.State.RemovePrivateAs)
 
 		// AS_PATH handling
-		path.PrependAsn(peer.Config.LocalAs, 1)
+		confed := peer.IsConfederationMember(global)
+		path.PrependAsn(peer.Config.LocalAs, 1, confed)
+		if !confed {
+			path.removeConfedAs()
+		}
 
 		// MED Handling
 		if med := path.getPathAttr(bgp.BGP_ATTR_TYPE_MULTI_EXIT_DISC); med != nil && !path.IsLocal() {
@@ -283,7 +254,7 @@ func UpdatePathAttrs(global *config.Global, peer *config.Neighbor, info *PeerInf
 		// if the path generated locally set local address as nexthop.
 		// if not, don't modify it.
 		// TODO: NEXT-HOP-SELF support
-		if path.IsLocal() && isZero(nexthop) {
+		if path.IsLocal() && nexthop.IsUnspecified() {
 			path.SetNexthop(localAddress)
 		}
 
@@ -291,7 +262,7 @@ func UpdatePathAttrs(global *config.Global, peer *config.Neighbor, info *PeerInf
 		// if the path has AS_PATH path attribute, don't modify it.
 		// if not, attach *empty* AS_PATH path attribute.
 		if nh := path.getPathAttr(bgp.BGP_ATTR_TYPE_AS_PATH); nh == nil {
-			path.PrependAsn(0, 0)
+			path.PrependAsn(0, 0, false)
 		}
 
 		// For iBGP peers we are required to send local-pref attribute
@@ -664,7 +635,12 @@ func (path *Path) GetAsString() string {
 
 			sep := " "
 			switch segment.Type {
-			case bgp.BGP_ASPATH_ATTR_TYPE_SET, bgp.BGP_ASPATH_ATTR_TYPE_CONFED_SET:
+			case bgp.BGP_ASPATH_ATTR_TYPE_CONFED_SEQ:
+				s.WriteString("(")
+			case bgp.BGP_ASPATH_ATTR_TYPE_CONFED_SET:
+				s.WriteString("[")
+				sep = ","
+			case bgp.BGP_ASPATH_ATTR_TYPE_SET:
 				s.WriteString("{")
 				sep = ","
 			}
@@ -675,7 +651,11 @@ func (path *Path) GetAsString() string {
 				}
 			}
 			switch segment.Type {
-			case bgp.BGP_ASPATH_ATTR_TYPE_SET, bgp.BGP_ASPATH_ATTR_TYPE_CONFED_SET:
+			case bgp.BGP_ASPATH_ATTR_TYPE_CONFED_SEQ:
+				s.WriteString(")")
+			case bgp.BGP_ASPATH_ATTR_TYPE_CONFED_SET:
+				s.WriteString("]")
+			case bgp.BGP_ASPATH_ATTR_TYPE_SET:
 				s.WriteString("}")
 			}
 		}
@@ -712,8 +692,38 @@ func (path *Path) getAsListofSpecificType(getAsSeq, getAsSet bool) []uint32 {
 	return asList
 }
 
+func (path *Path) GetLabelString() string {
+	label := ""
+	switch n := path.GetNlri().(type) {
+	case *bgp.LabeledIPAddrPrefix:
+		label = n.Labels.String()
+	case *bgp.LabeledIPv6AddrPrefix:
+		label = n.Labels.String()
+	case *bgp.LabeledVPNIPAddrPrefix:
+		label = n.Labels.String()
+	case *bgp.LabeledVPNIPv6AddrPrefix:
+		label = n.Labels.String()
+	case *bgp.EVPNNLRI:
+		switch route := n.RouteTypeData.(type) {
+		case *bgp.EVPNEthernetAutoDiscoveryRoute:
+			label = fmt.Sprintf("[%d]", route.Label)
+		case *bgp.EVPNMacIPAdvertisementRoute:
+			var l []string
+			for _, i := range route.Labels {
+				l = append(l, strconv.Itoa(int(i)))
+			}
+			label = fmt.Sprintf("[%s]", strings.Join(l, ","))
+		case *bgp.EVPNIPPrefixRoute:
+			label = fmt.Sprintf("[%d]", route.Label)
+		}
+	}
+	return label
+}
+
 // PrependAsn prepends AS number.
 // This function updates the AS_PATH attribute as follows.
+// (If the peer is in the confederation member AS,
+//  replace AS_SEQUENCE in the following sentence with AS_CONFED_SEQUENCE.)
 //  1) if the first path segment of the AS_PATH is of type
 //     AS_SEQUENCE, the local system prepends the specified AS num as
 //     the last element of the sequence (put it in the left-most
@@ -732,7 +742,13 @@ func (path *Path) getAsListofSpecificType(getAsSeq, getAsSet bool) []uint32 {
 //  3) if the AS_PATH is empty, the local system creates a path
 //     segment of type AS_SEQUENCE, places the specified AS number
 //     into that segment, and places that segment into the AS_PATH.
-func (path *Path) PrependAsn(asn uint32, repeat uint8) {
+func (path *Path) PrependAsn(asn uint32, repeat uint8, confed bool) {
+	var segType uint8
+	if confed {
+		segType = bgp.BGP_ASPATH_ATTR_TYPE_CONFED_SEQ
+	} else {
+		segType = bgp.BGP_ASPATH_ATTR_TYPE_SEQ
+	}
 
 	original := path.GetAsPath()
 
@@ -750,7 +766,7 @@ func (path *Path) PrependAsn(asn uint32, repeat uint8) {
 
 	if len(asPath.Value) > 0 {
 		fst := asPath.Value[0].(*bgp.As4PathParam)
-		if fst.Type == bgp.BGP_ASPATH_ATTR_TYPE_SEQ {
+		if fst.Type == segType {
 			if len(fst.AS)+int(repeat) > 255 {
 				repeat = uint8(255 - len(fst.AS))
 			}
@@ -761,7 +777,7 @@ func (path *Path) PrependAsn(asn uint32, repeat uint8) {
 	}
 
 	if len(asns) > 0 {
-		p := bgp.NewAs4PathParam(bgp.BGP_ASPATH_ATTR_TYPE_SEQ, asns)
+		p := bgp.NewAs4PathParam(segType, asns)
 		asPath.Value = append([]bgp.AsPathParamInterface{p}, asPath.Value...)
 	}
 	path.setPathAttr(asPath)
@@ -800,6 +816,21 @@ func (path *Path) RemovePrivateAS(localAS uint32, option config.RemovePrivateAsO
 	return
 }
 
+func (path *Path) removeConfedAs() {
+	original := path.GetAsPath()
+	if original == nil {
+		return
+	}
+	newAsParams := make([]bgp.AsPathParamInterface, 0, len(original.Value))
+	for _, v := range original.Value {
+		p := v.(*bgp.As4PathParam)
+		if p.Type == bgp.BGP_ASPATH_ATTR_TYPE_SEQ || p.Type == bgp.BGP_ASPATH_ATTR_TYPE_SET {
+			newAsParams = append(newAsParams, p)
+		}
+	}
+	path.setPathAttr(bgp.NewPathAttributeAsPath(newAsParams))
+}
+
 func (path *Path) ReplaceAS(localAS, peerAS uint32) *Path {
 	original := path.GetAsPath()
 	if original == nil {
@@ -1209,3 +1240,11 @@ func (p *Path) ToLocal() *Path {
 	path.IsNexthopInvalid = p.IsNexthopInvalid
 	return path
 }
+
+func (p *Path) SetHash(v uint32) {
+	p.attrsHash = v
+}
+
+func (p *Path) GetHash() uint32 {
+	return p.attrsHash
+}
