@@ -328,28 +328,29 @@ const (
 )
 
 var routeTypeValueMap = map[string]ROUTE_TYPE{
-	"system":         ROUTE_SYSTEM,
-	"kernel":         ROUTE_KERNEL,
-	"connect":        ROUTE_CONNECT,
-	"static":         ROUTE_STATIC,
-	"rip":            ROUTE_RIP,
-	"ripng":          ROUTE_RIPNG,
-	"ospf":           ROUTE_OSPF,
-	"ospf3":          ROUTE_OSPF6,
-	"isis":           ROUTE_ISIS,
-	"bgp":            ROUTE_BGP,
-	"pim":            ROUTE_PIM,
-	"hsls":           ROUTE_HSLS,
-	"olsr":           ROUTE_OLSR,
-	"babel":          ROUTE_BABEL,
-	"table":          FRR_ROUTE_TABLE,
-	"ldp":            FRR_ROUTE_LDP,
-	"vnc":            FRR_ROUTE_VNC,
-	"vnc-direct":     FRR_ROUTE_VNC_DIRECT,
-	"vnc-direct-rh":  FRR_ROUTE_VNC_DIRECT_RH,
-	"bgp-direct":     FRR_ROUTE_BGP_DIRECT,
-	"bgp-direct-ext": FRR_ROUTE_BGP_DIRECT_EXT,
-	"all":            FRR_ROUTE_ALL,
+	"system":             ROUTE_SYSTEM,
+	"kernel":             ROUTE_KERNEL,
+	"connect":            ROUTE_CONNECT, // hack for backyard compatibility
+	"directly-connected": ROUTE_CONNECT,
+	"static":             ROUTE_STATIC,
+	"rip":                ROUTE_RIP,
+	"ripng":              ROUTE_RIPNG,
+	"ospf":               ROUTE_OSPF,
+	"ospf3":              ROUTE_OSPF6,
+	"isis":               ROUTE_ISIS,
+	"bgp":                ROUTE_BGP,
+	"pim":                ROUTE_PIM,
+	"hsls":               ROUTE_HSLS,
+	"olsr":               ROUTE_OLSR,
+	"babel":              ROUTE_BABEL,
+	"table":              FRR_ROUTE_TABLE,
+	"ldp":                FRR_ROUTE_LDP,
+	"vnc":                FRR_ROUTE_VNC,
+	"vnc-direct":         FRR_ROUTE_VNC_DIRECT,
+	"vnc-direct-rh":      FRR_ROUTE_VNC_DIRECT_RH,
+	"bgp-direct":         FRR_ROUTE_BGP_DIRECT,
+	"bgp-direct-ext":     FRR_ROUTE_BGP_DIRECT_EXT,
+	"all":                FRR_ROUTE_ALL,
 }
 
 func RouteTypeFromString(typ string) (ROUTE_TYPE, error) {
